#define TOUCH1        7
#define MOTORB_1    9
#define MOTORB_2    10
#define MOTORA_1    5
#define MOTORA_2    6

#define BRAKE       300
#define BOTH        3
#define THRSH       500

#define RGB1_R  12    // 12
#define RGB1_G  11   // 11
#define RGB1_B  8   // 8
#define RGB2_R  16   // 16
#define RGB2_G  15   // 15
#define RGB2_B  14   // 14

#define TRIG 2
#define ECHO 3

#define BUZZER 13

//analog
#define LINELEFT  5
#define LINERIGHT  6
#define LINEMIDDLE  7
#define BAT    4

static int led[2][3] = { { RGB1_R, RGB1_G, RGB1_B }, { RGB2_R, RGB2_G, RGB2_B } }; 

void test_touch() 
{
  Serial.print("touch=");
  Serial.println(digitalRead(TOUCH1));
}

void show_light(uint8_t which_led, uint8_t r, uint8_t g, uint8_t b)
{
    if (which_led & 1)
    {
       digitalWrite(led[0][0], r);      
       digitalWrite(led[0][1], g);      
       digitalWrite(led[0][2], b);
    }
    if (which_led & 2)
    {
       digitalWrite(led[1][0], r);      
       digitalWrite(led[1][1], g);      
       digitalWrite(led[1][2], b);
    }
}

void test_rgb()
{
   for (int i = 0; i < 8; i++)
   {
     for (int le = 0; le < 3; le++)
     {
       show_light(le + 1, i & 1, (i >> 1) & 1, (i >> 2) & 1);      
       delay(300);
     }
   }
}

int distance()
{
  unsigned long time1;
  unsigned long time2;
  digitalWrite(TRIG, HIGH);
  delay(1);
  digitalWrite(TRIG,LOW);
  while (digitalRead(ECHO)== LOW);
  time1= micros();
  while (digitalRead(ECHO)==HIGH);
  time2=micros();
  return (time2-time1)/58;    
}

void test_ultrasonic() 
{
  Serial.print("DIST=");
  Serial.println(distance());  
}

int battery()
{
  analogReference(INTERNAL);
  analogRead(BAT);   // the internal 1.1V regulator was just turned on, first one or two readings will not be accurate
  delay(1200);
  analogRead(BAT);
  analogRead(BAT);    
  unsigned long bat_level = analogRead(BAT);
  if (bat_level * 121 / 1023 < 650)
  {
    motor(BOTH, 0);
    show_light(3, 0, 0, 0);
    while (1)
    {
      tone(BUZZER, 1760, 100);
      delay(500);
    }
  }
}

void test_battery() 
{
  analogReference(INTERNAL);
  analogRead(BAT);   // the internal 1.1V regulator was just turned on, first one or two readings will not be accurate
  delay(1200);
  analogRead(BAT);
  analogRead(BAT);
  
  Serial.print("raw=");
  unsigned long bat_level = analogRead(BAT);
  analogReference(DEFAULT);

  Serial.println(bat_level);
  Serial.print("voltage=");
  Serial.print(bat_level * 121 / 1023);
  Serial.println(" dV");
}

void test_sound() 
{
  tone(BUZZER, 440, 300);
  delay(300);
  tone(BUZZER, 880, 300);
  delay(300);
  tone(BUZZER, 1760, 300);
  delay(300);
}

void test_line() 
{
  Serial.print("LINE=(");
  Serial.print(analogRead(LINELEFT));
  Serial.print(",");
  Serial.print(analogRead(LINERIGHT));
  Serial.print(",");
  Serial.print(analogRead(LINEMIDDLE));
  Serial.println(")");
}
/*
motor A and B:
10  fwd
01  bwd
00  free
11  brake
*/
// which == 1  => motorA
// which == 2  => motorB
// which == 3  => BOTH
// speed == BRAKE  => active brake 
// speed == 0      => free run, no power
// speed <- 1..255   => fwd
// speed <- -255..-1  => bwd 
void motor(int which, int speed)
{
   int m1 = 0, m2 = 0;

   if (which == 1)
   {
      m1 = MOTORA_1;
      m2 = MOTORA_2;
   }
   else if (which == 2)
   {
      m1 = MOTORB_1;
      m2 = MOTORB_2;
   }
   else
   {
     motor(1, speed);
     motor(2, speed);
   }

    if (speed==BRAKE)
    {
        digitalWrite(m1, 1);
        digitalWrite(m2, 1);
    }
    else if (speed==0)
    {
        digitalWrite(m1, 0);
        digitalWrite(m2, 0);
    }
    else if (speed > 0)
    {
        digitalWrite(m1, 1);
        analogWrite(m2, 255 - speed);
    }
    else 
    {
        analogWrite(m1, 255 + speed);
        digitalWrite(m2, 1);
    }
}

void forward()
{
motor(BOTH, 255);
}

void slow()
{
motor(BOTH, 150);
}

void backward()
{
motor(BOTH, -255);  
}

void turnright()
{
    motor(1, 150);
    motor(2, -150); 
}

void turnleft()
{
    motor(1, -150);
    motor(2, 150); 
}

void right()
{
    motor(1, 150);
    motor(2, 20);  
}

void left()
{
    motor(1, 20);
    motor(2, 150);    
}

void stop()
{
    motor(BOTH, 0);
}


void brake()
{
    motor(BOTH, BRAKE);
}

void linefollower()
{
   for (int i = 0; i < 8; i++)
   {
     for (int le = 0; le < 3; le++)
     {
         show_light(le + 1, i & 1, (i >> 1) & 1, (i >> 2) & 1);      
         delay(30);
      }
    }
  show_light(3, 0, 0, 1);      

  while(digitalRead(TOUCH1)!=1)
  {
    if(distance()>10)
    {
     if(analogRead(LINELEFT)<THRSH)
     {
       left();
     }
        else  if(analogRead(LINERIGHT)<THRSH)
     {
       right();
     }
          else
     {
       forward();
     }
     }
     else 
     {
       avoid();
     }
     
  }
  stop();     
  while (digitalRead(TOUCH1) == 1) {}
  delay(100);
}

void avoid()
{
  show_light(3, 1, 0, 0);      
    turnleft();
    delay(300);
    forward();
    delay(700);
    turnright();
    delay(300);
    forward();
    delay(600);
    turnright();
    delay(300);
    slow();
        
    while (analogRead(LINEMIDDLE)>THRSH) {}
    show_light(3, 0, 1, 0);      
  
    turnleft();
    delay(200);
    while(analogRead(LINERIGHT)>THRSH){}
    turnright();
    delay(50);    
    show_light(3, 0, 0, 1);      
    forward();

    
}

void explorer()
{ 
  
  int dst=10;

  while(digitalRead(TOUCH1)!=1){
  while (distance()>dst)
  {
  forward();
  }
  backward();
  delay(300);
  turnleft();
  delay(300);
}
  stop();
   
}

void explorer2()
{  
  forward();
  while(digitalRead(TOUCH1)!=1)
  {
    if (distance() < 15)
    {
      backward();
      delay(300);
      turnleft();
      delay(300);
      forward();
    }
  }
  stop();   
}


void test_movements() 
{
  Serial.println("testing motor...");/*
  Serial.println("1...");
  digitalWrite(MOTORA_1, 1);
  digitalWrite(MOTORA_2, 1);
  delay(1000);*/
  Serial.println("2...");
  digitalWrite(MOTORB_1, 1);
  digitalWrite(MOTORB_2, 0);
  digitalWrite(MOTORA_1, 1);
  digitalWrite(MOTORA_2, 0);
  delay(1000);
  Serial.println("3...");
  digitalWrite(MOTORB_1, 0);
  digitalWrite(MOTORB_2, 1);
  digitalWrite(MOTORA_1, 0);
  digitalWrite(MOTORA_2, 1);
  delay(1000);
  Serial.println("4...");
  digitalWrite(MOTORB_1, 0);
  digitalWrite(MOTORB_2, 0);
  digitalWrite(MOTORA_1, 0);
  digitalWrite(MOTORA_2, 0);
  delay(1000);
    /*
  for (int i = 0; i < 256; i++)
  {
    analogWrite(MOTORB_2, i);
    delay(20);
  }
  digitalWrite(MOTORB_1, 0);
  */
  Serial.println("finished testing motor...");
  
}

void test_motors()
{
   for (int spd = 0; spd < 255; spd++)
   {
     motor(BOTH, spd);
     delay(10);
   }
   for (int spd = 255; spd > 0; spd--)
   {
     motor(BOTH, spd);
     delay(10);
   }
   for (int spd = 0; spd > -255; spd--)
   {
     motor(BOTH, spd);
     delay(10);
   }
   for (int spd = -255; spd < 0; spd++)
   {
     motor(BOTH, spd);
     delay(10);
   }

   // test braking
    motor(BOTH, 200);
    delay(1000);
    motor(BOTH, BRAKE);
    delay(300);
    motor(BOTH, 0);
    
    // test turning right and then left
    motor(1, 200);
    motor(2, -200);
    delay(1500);
    motor(1, -200);
    motor(2, 200);
    delay(1500);
    motor(BOTH, 0);   
}


void demo()
{
  // prints out menu and asks user for selection and then
  // runs test for a particular connected device
  char c = 0;
  do {
    Serial.println("1) test touch");
    Serial.println("2) test RGB");
    Serial.println("3) test ultrasonic");
    Serial.println("4) test battery");
    Serial.println("5) test sound");
    Serial.println("6) test line sensor");
    Serial.println("7) test movements");
    Serial.println("8) test motor");
    Serial.println("9) test explorer");
    Serial.println("A) test line");
    Serial.println("0) leave demo and start the main program");
    while (!Serial.available()) { 
      //battery();
      if (digitalRead(TOUCH1) == 1)  // user holds the TOUCH sensor => we run demo
      {
        while (digitalRead(TOUCH1) == 1) {}
        delay(100);
        linefollower();
      }
    }
    c = Serial.read();
    switch (c) {
      case '1': test_touch(); break;
      case '2': test_rgb(); break;
      case '3': test_ultrasonic(); break;
      case '4': test_battery(); break;
      case '5': test_sound(); break;
      case '6': test_line(); break;
      case '7': test_movements(); break;
      case '8': test_motors(); break;
      case '9': explorer2(); break;
      case 'A': linefollower(); break;       
    }  
  } while (c != '0');
}


void setup() {
   pinMode(TOUCH1, INPUT);
   pinMode(MOTORB_1, OUTPUT);
   pinMode(MOTORB_2, OUTPUT);
   pinMode(MOTORA_1, OUTPUT);
   pinMode(MOTORA_2, OUTPUT);
   pinMode(BUZZER, OUTPUT);
   pinMode(RGB1_R, OUTPUT);
   pinMode(RGB1_G, OUTPUT);
   pinMode(RGB1_B, OUTPUT);
   pinMode(RGB2_R, OUTPUT);
   pinMode(RGB2_G, OUTPUT);
   pinMode(RGB2_B, OUTPUT);
   pinMode(TRIG, OUTPUT);
   pinMode(ECHO, INPUT);
   Serial.begin(9600);
   demo();
}

void loop() {
  // put your main code here, to run repeatedly:
  show_light(3, 1, 0, 0);
  delay(300);
  show_light(3, 0, 1, 0);
  delay(300);
  show_light(3, 0, 0, 1);
  delay(1000);
}

